"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processMarkdown = processMarkdown;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const MarkdownParser_1 = require("./MarkdownParser"); // Update with your actual parser
const defaultOptions = {
    recursive: false,
    copyAssets: false,
    markdownExtensions: ['.md'],
};
async function processMarkdown(inputDir, outputDir, options) {
    const mergedOptions = { ...defaultOptions, ...options };
    const logger = mergedOptions.logger || {
        info: console.log,
        error: (err, msg) => console.error(msg, err),
    };
    // Validate input directory
    if (!fs_1.default.existsSync(inputDir)) {
        throw new Error(`Input directory does not exist: ${inputDir}`);
    }
    // Create output directory structure
    await fs_1.default.promises.mkdir(outputDir, { recursive: true });
    // Get all files respecting recursive option
    const files = await getFiles(inputDir, mergedOptions.recursive);
    // Process files in parallel with error handling
    await Promise.all(files.map(async (filePath) => {
        try {
            const relativePath = path_1.default.relative(inputDir, filePath);
            const outputPath = path_1.default.join(outputDir, relativePath);
            const outputDirPath = path_1.default.dirname(outputPath);
            // Create output directory structure for this file
            await fs_1.default.promises.mkdir(outputDirPath, { recursive: true });
            // Check if file is Markdown
            const isMarkdown = mergedOptions.markdownExtensions?.some(ext => filePath.toLowerCase().endsWith(ext.toLowerCase()));
            if (isMarkdown) {
                // Process Markdown file
                const markdownContent = await fs_1.default.promises.readFile(filePath, 'utf-8');
                const parser = new MarkdownParser_1.MarkdownParser();
                const htmlContent = parser.render(markdownContent);
                const finalHtml = mergedOptions.template
                    ? mergedOptions.template(htmlContent, filePath)
                    : htmlContent;
                // Change extension to .html
                const newPath = outputPath.replace(new RegExp(`${path_1.default.extname(filePath)}$`), '.html');
                await fs_1.default.promises.writeFile(newPath, finalHtml, 'utf-8');
                logger.info?.(`Converted: ${filePath} → ${newPath}`);
            }
            else if (mergedOptions.copyAssets) {
                // Copy non-Markdown file
                await fs_1.default.promises.copyFile(filePath, outputPath);
                logger.info?.(`Copied: ${filePath} → ${outputPath}`);
            }
        }
        catch (err) {
            logger.error?.(err, `Error processing ${filePath}`);
        }
    }));
}
/** Recursively get all files in a directory */
async function getFiles(dir, recursive = true) {
    try {
        const entries = await fs_1.default.promises.readdir(dir, { withFileTypes: true });
        const files = await Promise.all(entries.map(async (entry) => {
            const fullPath = path_1.default.join(dir, entry.name);
            if (entry.isDirectory()) {
                return recursive ? getFiles(fullPath, recursive) : [];
            }
            return [fullPath];
        }));
        return files.flat();
    }
    catch (error) {
        return [];
    }
}
